import argparse
import builtins
import os

_PYCACHE_DIR_NAME = '__pycache__'
_PYCACHE_FILE_NAME_EXT = '.pyc'
_CONFIRM_REMOVE_FILE_DEFAULT = lambda fn: (input(f'Do you want to remove file {repr(fn)}? [y/n] ') == 'y')

def del_cache_dir(d:str,
                  confirm=_CONFIRM_REMOVE_FILE_DEFAULT,
                  debug=False):

    print = (lambda *a,**ka: None) if not debug else builtins.print
    print(f'Directory: {d}')
    to_delete_dir = True
    for fn in os.listdir(d):

        if fn.endswith(_PYCACHE_FILE_NAME_EXT):

            fn_full = os.path.join(d, fn)
            if confirm(fn_full):

                print(f'   Remove file {fn_full}')
                os.remove(fn_full)

    if not os.listdir(d):

        print(f'Remove empty directory {d}')
        os.rmdir(d)

def do_it(wd       :str,
          recursive    =False,
          confirm      =_CONFIRM_REMOVE_FILE_DEFAULT,
          debug        =False):
    
    if not recursive:

        del_cache_dir(wd)
        return
    
    for dp,dnn,fnn in os.walk(wd):

        for dn in dnn:

            if dn == _PYCACHE_DIR_NAME:

                del_cache_dir(os.path.join(dp, dn), confirm=confirm, debug=debug)

def main():

    ap = argparse.ArgumentParser(description=f'Delete Python cache ({repr(_PYCACHE_DIR_NAME)})')
    class A:

        WORKING_DIR = 'wd'
        RECURSIVE   = 'r'
        NO_ASKING   = 'noask'
        DEBUG       = 'debug'

    ap.add_argument(f'--{A.WORKING_DIR}',
                    help='working directory, from which to search and remove cache')
    ap.add_argument(f'--{A.RECURSIVE}',
                    help='find and remove cache recursively i.e. in sub-directories',
                    action='store_true')
    ap.add_argument(f'--{A.NO_ASKING}',
                    help='do not ask before deleting\nMake sure that you do want to delete ALL cache files, before setting this options.',
                    action='store_true')
    ap.add_argument(f'--{A.DEBUG}',
                    help='debug i.e. print more information about what is happening',
                    action='store_true')
    # read args
    get = ap.parse_args().__getattribute__
    wd     = get(A.WORKING_DIR)
    r      = get(A.RECURSIVE)
    no_ask = get(A.NO_ASKING)
    debug  = get(A.DEBUG)
    # do it
    do_it(wd       =wd if wd is not None else os.getcwd(),
          recursive=r,
          confirm  =_CONFIRM_REMOVE_FILE_DEFAULT if not no_ask else (lambda fn: True),
          debug    =debug)

if __name__ == '__main__': main()


import argparse
import json
import subprocess
import sys

def do_it(args      :str|list[str],
          is_command:bool,
          indent    :int|None):
    
    if isinstance(args,str):
        args = [args,]
    if not is_command:
        out=args[0]
    else:
        out = subprocess.run(args,shell=True,capture_output=True).stdout.decode()

    try:
        return json.dumps(json.loads(out), indent=indent) if out.strip() else out
    except:
        return f'Could not parse as JSON\nOriginal:\n{out}'

def main(indent:int|None=2):

    class A:
        ARGS        = 'args'
        COMMAND     = 'c'
        INTERACTIVE = 'i'
    ap = argparse.ArgumentParser(description='JSON beautifier for literals and command outputs')
    ap.add_argument(f'--{A.COMMAND}', 
                    help='interpret argument(s) as a command',
                    action='store_true')
    ap.add_argument(f'{A.ARGS}', 
                    help='arguments - JSON literal or command token(s)',
                    nargs='*')
    ap.add_argument(f'--{A.INTERACTIVE}',
                    help='interactive input - useful if you want to paste the JSON literal without escaping any characters (quotes, etc)',
                    action='store_true')
    # parse args
    get = ap.parse_args().__getattribute__
    # do it
    print(do_it(args      =get(A.ARGS) if not get(A.INTERACTIVE) else input('Enter JSON literal: '),
                is_command=get(A.COMMAND),
                indent    =indent))
    

if __name__ == '__main__': main()



import argparse

def do_it(i:str):

    return f'"{i.replace("\\", "\\\\").replace("\"", "\\\"")}"'

if __name__ == "__main__":
    
    ap = argparse.ArgumentParser(description="Escape an input to be used as the body in a curl request (e.g. a JSON object with lots of nasty quotes)")
    class A:
        INPUT = "i"
    ap.add_argument(f'--{A.INPUT}',
                    help='input')
    get = ap.parse_args().__getattribute__
    # read args
    i = get(A.INPUT)
    # do it
    print(do_it(i if i is not None else input("Input: ")))



import os
import os.path
import re

def do_it(wd:str,
          fn:str|re.Pattern):

    if not isinstance(fn,re.Pattern):
        fn = re.compile(re.escape(fn))
    print(f'Looking in {wd}, where file names match {str(fn)}')
    found:list[str] = []
    for dp,dns,fns in os.walk(wd):

        for fn_ in fns:

            fn_full  = os.path.join(dp, fn_)
            if not fn.search(fn_full): continue
            found.append(fn_full)
    
    if found:

        for fn in found:
                
            print('>>> ' + fn)
    
    else:

        print('No occurrences.')

def main():

    import argparse

    class A:

        WORKING_DIRECTORY = 'wd'
        FILE_NAME         = 'f'
        AS_REGEX          = 're'

    p = argparse.ArgumentParser(description='Find all files whose name matches a given expression')
    p.add_argument(f'--{A.WORKING_DIRECTORY}',
                   help='working directory - defaults to current',
                   default='.')
    p.add_argument(f'{A.FILE_NAME}',
                   help='file name (full or partial) to search for')
    p.add_argument(f'--{A.AS_REGEX}',
                   help='search as regex',
                   action='store_true')
    get = p.parse_args().__getattribute__
    # do it
    do_it(wd=get(A.WORKING_DIRECTORY),
          fn=(lambda v: (v if not get(A.AS_REGEX) else re.compile(v)))(get(A.FILE_NAME)))

if __name__ == '__main__': main()



SAFE          = lambda r: f'{r}AAA583B9B0954BF58F3A9B6DA53F2DE9'
SWITCH        = 'SWITCH'
SWITCH_END    = 'END'
COMMENT_BEGIN = '<!--'
COMMENT_END   = '-->'
OFF_BEGIN     = f'{COMMENT_BEGIN}OFF:'
OFF_END       = f'{COMMENT_END}'
ENABLE        = lambda line: line if not line.startswith(OFF_BEGIN)                 else line[len(OFF_BEGIN):-len(OFF_END)]
DISABLE       = lambda line: line if not line.strip() or line.startswith(OFF_BEGIN) else f'{OFF_BEGIN}{line}{OFF_END}'

import re
import typing

class Feedback:

    def __init__(self):

        self.enabled :set[str] = set()
        self.disabled:set[str] = set()

def do_it(fn     :str,
          enable :typing.Callable[[str],bool],
          disable:typing.Callable[[str],bool]):
    
    with open(fn, 'r', encoding='utf-8') as f: 
        
        source = f.read()
    
    fb = Feedback()
    def repl(cases:str,
             block:str):

        if cases is None: raise AssertionError()
        cases:list[str] = cases.split(',')
        cases_enabled  = list(filter(enable , cases))
        cases_disabled = list(filter(disable, cases))
        if   cases_enabled:
            
            fb.enabled.update(cases_enabled)
            return '\n'.join(map(ENABLE, block.split('\n')))

        elif cases_disabled:
            
            return '\n'.join(map(DISABLE, block.split('\n')))
        
        else: return block

    a = re.sub(pattern=''.join((
        
        f'(?s)',
        f'(?P<sw>'   +f'{re.escape(COMMENT_BEGIN)}\\s*{re.escape(SWITCH)}'   +f':(?P<cases>\\S+)'+f'\\s*{re.escape(COMMENT_END)})',
        f'(?P<swcontent>.*?)',
        f'(?P<swend>'+f'{re.escape(COMMENT_BEGIN)}\\s*{re.escape(SWITCH_END)}'                   +f'\\s*{re.escape(COMMENT_END)})',
        
    )), repl=lambda match: f'{match.group('sw')}{repl(match.group('cases'),match.group('swcontent'))}{match.group('swend')}', string=source)
    with open(fn, 'w', encoding='utf-8') as f:

        f.write(a)

    return fb

if __name__ == '__main__':

    import argparse

    p = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, 
                                description    =f'Switch markdown blocks ON or OFF in an XML file\nDeactivated blocks are enclosed in a comment {repr(OFF_BEGIN)}...{repr(OFF_END)}.')
    p.add_argument('f',
                   help   ='file to process')
    p.add_argument('--on',
                   help   ='blocks to enable (switch ON)',
                   nargs  ='+',
                   default=[])
    p.add_argument('--off',
                   help   ='blocks to disable (switch OFF)',
                   nargs  ='+',
                   default=[])
    args = p.parse_args()
    print(args.on)
    print(args.off)
    do_it(fn     =args.f,
          enable =set(args.on) .__contains__,
          disable=set(args.off).__contains__)



import argparse

def do_it(fp:str,
          encoding:str|None=None): 
    
    with open(fp, mode='r', encoding=encoding, newline='\r\n') as f:
        a:list[str] = f.readlines()
    with open(fp, mode='w', encoding=encoding, newline='\n')   as f:
        f.writelines(map(lambda line: (line if not line.endswith('\r\n') else line[:-2]+'\n'), a))

def main():

    ap = argparse.ArgumentParser(description="Be gone, carriage returns!!!",
                                 formatter_class=argparse.RawTextHelpFormatter)
    class A:
        FILE_PATH = 'f'
        ENCODING  = 'enc'
    ap.add_argument(f'{A.FILE_PATH}',
                    help='text file path from which to remove newline carriage returns')
    ap.add_argument(f'--{A.ENCODING}',
                    help='encoding (optional)')
    get = ap.parse_args().__getattribute__
    do_it(get(A.FILE_PATH), 
          encoding=get(A.ENCODING))

if __name__ == '__main__': main()



import argparse
import os

DELIMITER_DEFAULT = ';'

def get_as_list(name:str,
                delimiter=DELIMITER_DEFAULT):

    return os.environ[name].split(delimiter)

def main():

    ap = argparse.ArgumentParser(description='list the elements of a delimiter-based list environment variable')
    class A: 
        NAME = 'name'
        DELIMITER = 'delim'
        INDENT = 'indent'
        SHOW_EMPTY = 'showempty'
    class Defaults:
        DELIMITER = DELIMITER_DEFAULT
        INDENT = '  '

    ap.add_argument(f'{A.NAME}',
                    help='name of the environment variable to print')
    ap.add_argument(f'--{A.DELIMITER}',
                    help=f'delimiter\nDefault: {repr(Defaults.DELIMITER)}')
    ap.add_argument(f'--{A.INDENT}',
                    help=f'indentation to use when printing\nDefault: {repr(Defaults.INDENT)}')
    ap.add_argument(f'--{A.SHOW_EMPTY}',
                    help=f'show empty values',
                    action='store_true')
    # parse args
    get = ap.parse_args().__getattribute__
    name       = get(A.NAME)
    delimiter  = (lambda x: (x if x is not None else Defaults.DELIMITER))(get(A.DELIMITER))
    indent     = (lambda x: (x if x is not None else Defaults.INDENT))   (get(A.INDENT))
    show_empty = get(A.SHOW_EMPTY)
    # do it
    print('\n'.join(f'{indent}{v}' for v in get_as_list(name, delimiter=delimiter) if v or show_empty))

if __name__ == '__main__': main()


import argparse
import json
import subprocess
import sys

def do_it(flat  :str,
          indent:int|None):
    
    return json.dumps(json.loads(flat), indent=indent) if flat.strip() else flat

def main(indent:int|None=2):

    class A:
        ARGS        = 'args'
        COMMAND     = 'command'
        INTERACTIVE = 'interactive'
    ap = argparse.ArgumentParser(description='JSON beautifier for literals and command outputs')
    group = ap.add_mutually_exclusive_group()
    group.add_argument(f'--{A.COMMAND}','-c', 
                    help='interpret argument(s) as a command\nOtherwise, expect single argument as JSON literal',
                    action='store_true')
    group.add_argument(f'--{A.INTERACTIVE}','-i',
                    help='interactive input - useful if you want to pass the JSON literal without escaping any characters (quotes, etc)',
                    action='store_true')
    ap.add_argument(f'{A.ARGS}', 
                    help='arguments - JSON literal or command token(s)',
                    nargs='*')
    # parse args
    get = ap.parse_args().__getattribute__
    args:list[str] = get(A.ARGS)
    is_command:bool = get(A.COMMAND)
    interactive:bool = get(A.INTERACTIVE)
    # do it
    if not is_command:
        if not interactive:
            if len(args) != 1:
                raise Exception('Expected single argument as JSON literal')
            flat=args[0]
        else:
            flat=input('Enter JSON literal: ')
    else:
        flat= subprocess.run(args,
                             shell=True,
                             capture_output=True).stdout.decode()
    print(do_it(flat  =flat,
                indent=indent))

if __name__ == '__main__': main()



import argparse
import os
import subprocess
import typing

def do_it(wd:str,
          command:str|typing.Iterable[str],
          depth:int|None=1):
    
    if depth is not None and depth <= 0:
        return
    curwd = os.getcwd()
    os.chdir(wd)
    try: 
        for root, dirs, files in os.walk('.', topdown=True):
            for dir in dirs:
                try:
                    os.chdir(os.path.join(root, dir))
                except: continue
                try:
                    print(os.getcwd())
                    if isinstance(command, str):
                        os.system(command)
                    else:
                        subprocess.run(list(command), shell=True)
                    if depth is None or 1 < depth:
                        do_it(wd=os.getcwd(),
                            command=command,
                            depth=None if depth is None else depth - 1)
                finally:
                    os.chdir('..')
    finally:
        os.chdir(curwd)

def main():

    ap = argparse.ArgumentParser(description='Run command on directories recursively',
                                 formatter_class=argparse.RawTextHelpFormatter)
    class A:
        WD = 'wd'
        DEPTH = 'depth'
        NO_DEPTH = 'no-depth'
        COMMAND = 'cmd'
    class Defaults:
        DEPTH = 1
    ap.add_argument(f'--{A.WD}', 
                    help='working directory')
    depth_args = ap.add_mutually_exclusive_group()
    depth_args.add_argument(f'--{A.DEPTH}','-d',
                            type=int,
                            help=f'maximum recursion depth\nDefault: {repr(Defaults.DEPTH)}',
                            default=Defaults.DEPTH)
    depth_args.add_argument(f'--{A.NO_DEPTH}',
                            help='no recursion depth limit',
                            action='store_true')
    ap.add_argument(f'{A.COMMAND}', 
                    nargs=argparse.REMAINDER,
                    help='command to run on each directory')
    get = ap.parse_args().__getattribute__
    wd      :str  = get(A.WD) if get(A.WD) is not None else os.getcwd()
    command :typing.Iterable[str] = get(A.COMMAND)
    no_depth:bool = get(A.NO_DEPTH.replace('-','_'))
    depth   :int  = get(A.DEPTH)
    do_it(wd     =wd,
          command=command,
          depth  =depth if not no_depth else None)

if __name__ == '__main__': main()


import argparse
import os
import random
import subprocess

MARKER = b'8B1F2397EBFF4A07969BF2744340BAA3'
FILE_RW_CHUNK_SIZE = 1048*1048
FILE_ENCRYPTED_TAIL = '.JUMBLED'
FILE_DECRYPTED_TAIL = '.UNJUMBLED'

class Jumbler:

    def __init__(self,key:str):

        self._key    = key
        self._random = random.Random()

    def _jumble(self,f:str,fo:str,de:bool=False):

        self._random.seed(self._key)
        with open(f,mode='rb') as fr:

            with open(fo,mode='wb') as fw:

                if not de:

                    fw.write(MARKER)

                else:

                    fr.seek(len(MARKER))

                while True:

                    chunk = fr.read(FILE_RW_CHUNK_SIZE)
                    if not chunk: break
                    rbytes = self._random.randbytes(len(chunk))
                    fw.write(bytes(a^b for a,b in zip(chunk, rbytes)))

        return fo

    def jumble(self,f:str,fo:str|None=None):

        with open(f,mode='rb') as fr:

            if fr.read(len(MARKER)) == MARKER:
            
                raise Exception('file encrypted already')

        return self._jumble(f,fo if fo is not None else \
                            f'{f}{FILE_ENCRYPTED_TAIL}',de=False)

    def unjumble(self,f:str,fo:str|None=None):

        with open(f,mode='rb') as fr:

            if fr.read(len(MARKER)) != MARKER:
            
                raise Exception('file not encrypted - cannot decrypt')

        return self._jumble(f,fo                          if fo is not None                  else \
                            f[:-len(FILE_ENCRYPTED_TAIL)] if f.endswith(FILE_ENCRYPTED_TAIL) else \
                            f'{f}{FILE_DECRYPTED_TAIL}',de=True)

class Mode:  
    def __init__(self,descr:str): self._descr = descr
class Modes:
    PEEK = Mode('peek')
    EDIT = Mode('edit')

def do_it(f  :str,
          key:str,
          cleanup=False,
          de     =False,
          mode   :Mode|None=None):

    jumbler = Jumbler(key=key)
    if mode is Modes.PEEK:

        fo=jumbler.unjumble(f)
        subprocess.Popen([fo,],shell=True).wait()
        os.remove(fo)

    elif mode is Modes.EDIT:

        fo=jumbler.unjumble(f)
        subprocess.Popen([fo,],shell=True).wait()
        do_it(fo, key, cleanup=True)

    elif mode is None:
    
        (jumbler.jumble if not de else \
         jumbler.unjumble)(f)
        if cleanup:

            os.remove(f)

    else: raise AssertionError()

def main():

    class A:
        PATH    ='f'
        KEY     ='key'
        CLEANUP ='c'
        DECRYPT ='de'
        PEEK    ='peek'
        EDIT    ='edit'
    ap = argparse.ArgumentParser(description=f'Encrypt / Decrypt a file\nThe default action is to encrypt. To decrypt, set option {repr(A.DECRYPT)}.')
    ap.add_argument(f'{A.PATH}',
                    help='path to the file')
    ap.add_argument(f'{A.KEY}',
                    help='key (password)')
    ap.add_argument(f'--{A.CLEANUP}',
                    help='after processing, remove the original file',
                    action='store_true')
    ap.add_argument(f'--{A.DECRYPT}',
                    help='decrypt',
                    action='store_true')
    ap.add_argument(f'--{A.PEEK}',
                    help='decrypt and open the decrypted file with the system\'s default editor. After closing, the decrypted file will be removed.',
                    action='store_true',)
    ap.add_argument(f'--{A.EDIT}',
                    help='decrypt, open the decrypted file with the system\'s default editor, save and re-encrypt.\nThis option is useful for quickly editing an encrypted file.',
                    action='store_true',)
    get = ap.parse_args().__getattribute__
    mode = Modes.PEEK if get(A.PEEK) else \
           Modes.EDIT if get(A.EDIT) else \
           None
    if mode is not None:
        print(f'Mode: {mode._descr}')
    do_it(get(A.PATH),
          get(A.KEY),
          cleanup=get(A.CLEANUP),
          de=get(A.DECRYPT),
          mode=mode)
    print('Done')

if __name__ == '__main__': main()